from common.utils import *
from common.utilsgui import *

def render(dp, fields, vld, readonly):
	MSorder = fields.getint("MSorder", 0)
	MSrev = fields.getint("MSrev", 0)

	out = []
	out.append("""<div class="tab_title">Active mounts (parameters)</div>""")
	out.append("""<table class="acid_tab acid_tab_zebra_C1_C2 acid_tab_storageid_mfsmounts">""")
	out.append("""	<tr>""")
	out.append("""		<th rowspan="2" class="acid_tab_enumerate">#</th>""")
	out.append("""		<th rowspan="2" class="wrap">Session id</th>""")
	out.append("""		<th rowspan="2">Host</th>""")
	out.append("""		<th rowspan="2">IP</th>""")
	out.append("""		<th rowspan="2" class="wrap">Mount point</th>""")
	out.append("""		<th rowspan="2" class="wrap">Open files</th>""")
	out.append("""		<th rowspan="2" class="wrap">Number of connections</th>""")
	out.append("""		<th rowspan="2">Version</th>""")
	out.append("""		<th rowspan="2">Root dir</th>""")
	out.append("""		<th rowspan="2">RO/RW</th>""")
	out.append("""		<th rowspan="2" class="wrap">Restricted IP</th>""")
	out.append("""		<th rowspan="2" class="wrap">Ignore gid</th>""")
	out.append("""		<th rowspan="2">Admin</th>""")
	out.append("""		<th colspan="2">Map&nbsp;root</th>""")
	out.append("""		<th colspan="2">Map&nbsp;users</th>""")
	out.append("""		<th rowspan="2">Allowed&nbsp;sclasses</th>""")
	out.append("""		<th colspan="2">Trashretention&nbsp;limits</th>""")
	if dp.master().has_feature(FEATURE_EXPORT_UMASK):
		out.append("""		<th rowspan="2" class="wrap">Global umask</th>""")
	if dp.master().has_feature(FEATURE_EXPORT_DISABLES):
		out.append("""		<th rowspan="2" class="wrap">Disables mask</th>""")
	out.append("""	</tr>""")
	out.append("""	<tr>""")
	out.append("""		<th>uid</th>""")
	out.append("""		<th>gid</th>""")
	out.append("""		<th>uid</th>""")
	out.append("""		<th>gid</th>""")
	out.append("""		<th>min</th>""")
	out.append("""		<th>max</th>""")
	out.append("""	</tr>""")

	sessions,dsessions = dp.get_sessions_by_state(MSorder, MSrev)

	# Show active mounts
	for ses in sessions:
		out.append("""	<tr>""")
		out.append("""		<td align="right"></td>""")
		out.append("""		<td align="center">%s</td>""" % ses.get_sessionstr())
		out.append("""		<td align="left">%s</td>""" % ses.host)
		out.append("""		<td align="center"><span class="sortkey">%s </span>%s</td>""" % (ses.sortip,ses.strip))
		out.append("""		<td align="left">%s</td>""" % htmlentities(ses.info))
		out.append("""		<td align="center">%s</td>""" % decimal_number_html(ses.openfiles))
		out.append("""		<td align="center">%s</td>""" % decimal_number_html(ses.nsocks))
		issues = vld.check_mount_version(ses)
		out.append("""		<td align="center"><span class="sortkey">%s</span>%s</td>""" % (ses.sortver,issues.span(ses.strver.replace("PRO","<small>PRO</small>"))))
		out.append("""		<td align="left">%s</td>""" % (".&nbsp;(META)" if ses.meta else htmlentities(ses.path)))
		out.append("""		<td align="center">%s</td>""" % ("ro" if ses.sesflags&1 else "rw"))
		out.append("""		<td align="center">%s</td>""" % ("no" if ses.sesflags&2 else "yes"))
		out.append("""		<td align="center">%s</td>""" % ("-" if ses.meta else "yes" if ses.sesflags&4 else "no"))
		out.append("""		<td align="center">%s</td>""" % ("-" if ses.meta else "yes" if ses.sesflags&8 else "no"))
		if ses.meta:
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
		else:
			out.append("""		<td align="right">%u</td>""" % ses.rootuid)
			out.append("""		<td align="right">%u</td>""" % ses.rootgid)
		if ses.meta or (ses.sesflags&16)==0:
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
		else:
			out.append("""		<td align="right">%u</td>""" % ses.mapalluid)
			out.append("""		<td align="right">%u</td>""" % ses.mapallgid)
		out.append("""		<td align="center"><span class="sortkey">%u</span>%s</td>""" % (ses.get_sclassgroups_sort(),ses.get_sclassgroups_str()))
		if ses.mintrashretention!=None and ses.maxtrashretention!=None:
			out.append("""		<td align="right"><span class="sortkey">%u </span><a style="cursor:default" title="%s">%s</a></td>""" % (ses.mintrashretention,timeduration_to_fullstr(ses.mintrashretention),timeduration_to_shortstr(ses.mintrashretention)))
			out.append("""		<td align="right"><span class="sortkey">%u </span><a style="cursor:default" title="%s">%s</a></td>""" % (ses.maxtrashretention,timeduration_to_fullstr(ses.maxtrashretention),timeduration_to_shortstr(ses.maxtrashretention)))
		else:
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
			out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
		if dp.master().has_feature(FEATURE_EXPORT_UMASK):
			if ses.umaskval==None:
				out.append("""		<td align="center"><span class="sortkey">-1 </span>-</td>""")
			else:
				out.append("""		<td align="center">%03o</td>""" % ses.umaskval)
		if dp.master().has_feature(FEATURE_EXPORT_DISABLES):
			out.append("""		<td align="center"><span class="sortkey">%u </span><a style="cursor:default" title="%s">%08X</a></td>""" % (ses.disables,disablesmask_to_string(ses.disables),ses.disables))
		out.append("""	</tr>""")
	out.append("""</table>""")
	
	# Show inactive mounts
	if len(dsessions)>0:
		out.append("""<div class="tab_title" data-tt="mnt_inactive_list">Inactive mounts (parameters)</div>""")
		out.append("""<table class="acid_tab acid_tab_zebra_C1_C2 acid_tab_storageid_mfsmounts">""")
		out.append("""	<tr>""")
		out.append("""		<th class="acid_tab_enumerate">#</th>""")
		out.append("""		<th>Session&nbsp;id</th>""")
		out.append("""		<th>Host</th>""")
		out.append("""		<th>IP</th>""")
		out.append("""		<th>Mount&nbsp;point</th>""")
		out.append("""		<th>Open files</th>""")
		out.append("""		<th>Expires</th>""")
		if (not readonly):
			out.append("""		<th>cmd</th>""")
		out.append("""	</tr>""")
		
		issues = vld.issue('mnt_inactive')
		for ses in dsessions:
			out.append("""	<tr>""")
			out.append("""		<td align="right"></td>""")
			out.append("""		<td align="center">%s</td>""" % issues.span(str(ses.sessionid)))
			out.append("""		<td align="left">%s</td>""" % ses.host)
			out.append("""		<td align="center"><span class="sortkey">%s </span>%s</td>""" % (ses.sortip,ses.strip))
			out.append("""		<td align="left">%s</td>""" % ses.info)
			out.append("""		<td align="center">%u</td>""" % ses.openfiles)
			out.append("""		<td align="center">%s s</td>""" % decimal_number_html(ses.expire))
			if (not readonly):
				out.append("""		<td align="center"><a href="%s">click to remove</a></td>""" % fields.createhtmllink({"MSremove":("%u" % (ses.sessionid))}))
			out.append("""	</tr>""")
		out.append("""</table>""")

	return out


import time

from common.constants import *
from common.utils import *
from common.utilsgui import *

def render(dp, fields, vld):
	out = []
	lic = dp.get_licence()
	if lic:
		colnames = lic.licextrainfo.keys()
		if lic.licver==0:
			columns = 8
		else:
			columns = 7 + len(colnames)
		out.append("""<div class="tab_title">Licence info</div>""")
		out.append("""<table class="FR no-hover">""")
		out.append("""	<tr>""")
		if lic.licver>=1:
			out.append("""		<th>ID</th>""")
		out.append("""		<th>Issuer</th>""")
		if lic.licver==0:
			out.append("""		<th>User</th>""")
		else:
			out.append("""		<th>Client</th>""")
			for colname in colnames:
				out.append("""		<th>%s</th>""" % htmlentities(colname))
		out.append("""		<th>Type</th>""")
		out.append("""		<th>Max version</th>""")
		out.append("""		<th>Expires</th>""")
		if lic.licver==0:
			out.append("""		<th>Max chunkserver size</th>""")
			out.append("""		<th>Max number of chunkservers</th>""")
			out.append("""		<th>Quota</th>""")
		else:
			out.append("""		<th>Utilization (raw data)</th>""")
		out.append("""	</tr>""")
		out.append("""	<tr>""")

		if lic.licver>=1:
			out.append("""		<td>%s</td>""" % htmlentities(lic.licid))
		out.append("""		<td>%s</td>""" % htmlentities(lic.licissuer))
		out.append("""		<td>%s</td>""" % htmlentities(lic.licuser))
		if lic.licver>=1:
			for colname in colnames:
				out.append("""		<td>%s</td>""" % htmlentities(lic.licextrainfo[colname]))
		out.append("""		<td>%s</td>""" % htmlentities(lic.lictypestr))
		issues = vld.check_licence_version(lic, dp.master().version)
		out.append("""		<td>%s</td>""" % issues.span(lic.get_max_version_str()))
		timeissues = vld.check_licence_time(lic)
		if lic.is_time_unlimited():
			out.append("""		<td>never</td>""")
		else:
			if lic.licleft<0:
				expires_msg = " (expired)"
			else:
				expires_msg = """ <span data-tt-info="%s">%s</span>""" % (timeduration_to_fullstr(lic.licleft),timeduration_to_shortstr(lic.licleft))
			out.append("""		<td>%s</td>""" % timeissues.span(time.asctime(time.localtime(lic.licmaxtime))+expires_msg ))
		
		sizeissues = vld.check_licence_size(lic)
		if lic.licver==0:
			if lic.is_cs_size_unlimited():
				out.append("""		<td>unlimited</td>""")
			else:
				out.append("""		<td><span data-tt-info="%s B">%s</span></td>""" % (decimal_number(lic.csmaxsize),humanize_number(lic.csmaxsize,"&nbsp;")))
			if lic.is_cs_cnt_unlimited():
				out.append("""		<td>unlimited</td>""")
			else:
				out.append("""		<td>%u</td>""" % lic.csmaxcnt)
			if lic.quota:
				out.append("""		<td>YES</td>""")
			else:
				out.append("""		<td>NO</td>""")
		else:
			if lic.is_size_unlimited():
				out.append("""		<td>unlimited</td>""")
			else:
				size_msg = sizeissues.span("%s of %s (%.2f%%)" % (humanize_number(lic.currentsize,"&nbsp;"),humanize_number(lic.licmaxsize,"&nbsp;"),(100.0 * lic.currentsize / lic.licmaxsize)) )
				out.append("""		<td><span class="OK" data-tt-info="used: %s B, available: %s B">%s</span></td>""" % (decimal_number(lic.currentsize),decimal_number(lic.licmaxsize),size_msg))
		out.append("""	</tr>""")
		if lic.licver>=1:
			if len(lic.addinfo)>0:
				out.append("""	<tr><td colspan="%u">%s</td></tr>""" % (columns,htmlentities(lic.addinfo)))
			if sizeissues.contains('lic_exhausted'):
				out.append("""	<tr><td colspan="%u"><span class="ERROR">You have reached the raw space limit of your licence. This eventually results in your system becoming read-only. In order to expand your licence, please contact our team at <a href="mailto:contact@moosefs.com">contact@moosefs.com</a></span></td></tr>""" % columns)
			elif sizeissues.contains('lic_exhausting') or sizeissues.contains('lic_almost_exhausted'):
				out.append("""	<tr><td colspan="%u"><span class="WARNING">You are reaching the raw space limit of your licence. In order to expand it, please contact our team at <a href="mailto:contact@moosefs.com">contact@moosefs.com</a></span></td></tr>""" % columns)
			if timeissues.contains('lic_expiring'):
				out.append("""	<tr><td colspan="%u"><span class="WARNING">Your licence is about to expire. In order to extend it, please contact our team at <a href="mailto:contact@moosefs.com">contact@moosefs.com</a></span></td></tr>""" % columns)
			elif timeissues.contains('lic_expired'):
				out.append("""	<tr><td colspan="%u"><span class="ERROR">Your licence has expired. In order to extend it, please contact our team at <a href="mailto:contact@moosefs.com">contact@moosefs.com</a></span></td></tr>""" % columns)
		out.append("""</table>""")
	else:
		out.append("""<div class="tab_title ERROR">Oops!</div>""")
		out.append("""<table class="FR MESSAGE">""")
		out.append("""	<tr><td align="left"><span class="ERROR">Unrecognized answer from Master server</span></td></tr>""")
		out.append("""</table>""")

	return out


from common.constants import *
from common.utils import *
from common.utilsgui import *

def render(dp, fields, vld, selectable):	
	ICsclassid = fields.getint("ICsclassid", -1)
	(matrix,progressstatus)=dp.get_matrix(ICsclassid)
	# Get sums of missing, undergoal and endangered  (for all sclasses)
	(mx_summary,_)=dp.get_matrix_summary()
	issues_sum = vld.check_mx_summary(mx_summary)

	if len(matrix)==2 or len(matrix)==6 or len(matrix)==8:
		out = []
		out.append("""<form action="#">""")
		out.append("""<div class="tab_title">Chunk matrix table %s</div>""" % issues_sum.icon())
		out.append("""<table class="acid_tab acid_tab_storageid_mfsmatrix" id="mfsmatrix">""")
		out.append("""	<tr><th colspan="14">""") 
		out.append("""<div style="display: flex; align-items: center;">""")
		options=[(-45, "Regular chunks only", "'marked for removal' excluded","acid_tab.switchdisplay('mfsmatrix','matrixar_vis',1);"),
				(45, "All chunks","'marked for removal' included","acid_tab.switchdisplay('mfsmatrix','matrixar_vis',0);")]
		out.append(html_knob_selector("matrixar_vis",12,(380,42),(190,20),options))
		if len(matrix)==6:
			options=[(-45, "Copies and EC chunks", None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',0);"),
					(45, "Copies only",None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',1);"),
					(135, "EC chunks only",None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',2);")]
			out.append(html_knob_selector("matrixec_vis",12,(340,42),(170,20),options))
		elif len(matrix)==8:
			options=[(-45,"Copies and EC chunks", None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',0);"),
					(45,"EC8 chunks only",None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',2);"),
					(135,"EC4 chunks only",None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',3);"),
					(-135,"Copies only",None,"acid_tab.switchdisplay('mfsmatrix','matrixec_vis',1);")]
			out.append(html_knob_selector("matrixec_vis",12,(350,42),(175,20),options))

		
		if dp.master().has_feature(FEATURE_SCLASS_IN_MATRIX):
			sclass_dict = dp.get_used_sclasses_names()
		else:
			sclass_dict = {}
		if len(sclass_dict)>1 and selectable:
			sclass_used=[]
			name_too_long=False
			for sclassid in sclass_dict:
				if ICsclassid!=sclassid:
					if sclass_dict[sclassid].has_chunks:
						sclass_used.append((sclassid,sclass_dict[sclassid].name))
						if len(sclass_dict[sclassid].name)>17:
							name_too_long=True
				else:
					sclass_used.append((sclassid,sclass_dict[sclassid].name))
					if len(sclass_dict[sclassid].name)>17:
						name_too_long=True					
			sclass_count=len(sclass_used)
			if sclass_count<=5 and not name_too_long: #just a few (max 5+1=6) storage classes, use a knob 
				angles=[[-45],[-45,45],[-45,45,135],[-135,-45,45,135],[-135,-45,45,90,135],[-135,-90,-45,45,90,135]]
				options=[(angles[sclass_count][0],"All storage classes", None,"document.location.href='%s&ICsclassid=-1'" % fields.createrawlink({"ICsclassid":""}))]
				selected = 0 #default selection
				i=1
				for (sclassid, name) in sclass_used:
					options.append((angles[sclass_count][i],name, None,"document.location.href='%s&ICsclassid=%u'" % (fields.createrawlink({"ICsclassid":""}),sclassid)))
					selected = i if ICsclassid==sclassid else selected
					i+=1
				out.append(html_knob_selector("sclass_knob",12,(340,43),(170,21),options,True,selected))
			else: #too many classes to display as a knob, use a drop-down instead 
				out.append("""	<label for="storage_class_select">Storage class </label><div class="select-fl"><select id="storage_class_select" name="storage_class" onchange="document.location.href='%s&ICsclassid='+this.options[this.selectedIndex].value">""" % fields.createrawlink({"ICsclassid":""}))
				if ICsclassid>=0:
					out.append("""		<option value="-1">all storage classes</option>""")
				else:
					out.append("""		<option value="-1" selected="selected">all storage classes</option>""")
				for sclassid in sclass_dict:
					if ICsclassid!=sclassid:
						if sclass_dict[sclassid].has_chunks:
							out.append("""		<option value="%u">%s</option>""" % (sclassid,sclass_dict[sclassid].name))
					else:
						out.append("""		<option value="%u" selected="selected">%s</option>""" % (sclassid,sclass_dict[sclassid].name))
				if ICsclassid>=0 and ICsclassid not in sclass_dict:
					out.append("""		<option value="%u" selected="selected">nonexistent class number %u</option>""" % (ICsclassid,ICsclassid))
				out.append("""	</select><span class="arrow"></span></div>""")
		out.append("""</div>""")
		out.append("""	</th></tr>""")
		issues = vld.check_mx_progress(progressstatus)
		if issues.any():
			progressstr = "disconnections" if (progressstatus==1) else "connections" if (progressstatus==2) else "connections and disconnections"
			msg = "Counters may not be valid - %s in progress" % progressstr
			out.append("""	<tr><th colspan="14">%s</th></tr>""" % issues.span(msg))
		out.append("""	<tr>""")
		out.append("""		<th rowspan="2" colspan="2" class="acid_tab_skip">""")
		out.append("""		</th>""")
		out.append("""		<th colspan="12" class="acid_tab_skip">""")
		line = []
		line.append("""<span class="matrixec_vis0">Actual Redundancy Level</span>""")
		line.append("""<span class="matrixec_vis1">Actual number of full copies</span>""")
		line.append("""<span class="matrixec_vis2">Actual number of EC8 parts</span>""")
		if len(matrix)>=8:
			line.append("""<span class="matrixec_vis3">Actual number of EC4 parts</span>""")
		line.append("""<span class="matrixar_vis0">, 'marked for removal' included</span>""")
		line.append("""<span class="matrixar_vis1">, 'marked for removal' excluded</span>""")	
		line.append("""<span class="matrixec_vis0">&nbsp;<small>('RL = n' means EC: 8&#x202F;+n or 4&#x202F;+n&#x202F;parts, GOAL: 1&#x202F;+n&#x202F;copies)</small></span>""")
		out.append("".join(line))
		out.append("""		</th>""")
		out.append("""	</tr>""")
		out.append("""	<tr>""")
		for rl in range(11):
			out.append("""		<th class="acid_tab_skip" style="min-width:50px;">""")
			out.append("""			<span class="matrixec_vis0">%s</span>""" % ("missing" if (rl==0) else "RL&#x202F;>=&#x202F;9" if (rl==10) else ("RL&#x202F;=&#x202F;%u" % (rl-1))))
			out.append("""			<span class="matrixec_vis1">%s</span>""" % ("missing" if (rl==0) else ("1&#x202F;+%u&#x202F;copies" % (rl-1))))
			out.append("""			<span class="matrixec_vis2">%s</span>""" % ("missing" if (rl==0) else ("8&#x202F;+%u&#x202F;parts" % (rl-1))))
			if len(matrix)>=8:
				out.append("""			<span class="matrixec_vis3">%s</span>""" % ("missing" if (rl==0) else ("4&#x202F;+%u&#x202F;parts" % (rl-1))))
			out.append("""		</th>""")
		out.append("""		<th class="acid_tab_skip" style="min-width:50px;">all</th>""")
		out.append("""	</tr>""")

	classsum = []
	sumlist = []
	for i in range(len(matrix)):
		classsum.append(7*[0])
		sumlist.append(11*[0])
	left_col_once=1

	for goal in range(11):
		out.append("""	<tr>""")
		if left_col_once:
			vertical_title = """<svg viewbox="0 0 16 220" width="16" height="220" xmlns="http://www.w3.org/2000/svg"><text transform="rotate(-90)" style="font-family: arial, verdana, sans-serif; font-size: 13px; text-anchor: middle;" x="-110" y="12">PLACEHOLDER</text></svg>"""
			out.append("""<th rowspan="12" style="min-width: 16px;" class="acid_tab_skip">""")
			out.append("""			<div class="matrixec_vis0">%s</div>""" % vertical_title.replace("PLACEHOLDER", "Expected Redundancy Level (RL)"))
			out.append("""			<div class="matrixec_vis1">%s</div>""" % vertical_title.replace("PLACEHOLDER", "Expected number of copies"))
			out.append("""			<div class="matrixec_vis2">%s</div>""" % vertical_title.replace("PLACEHOLDER", "Expected number of parts"))
			if len(matrix)>=8:
				out.append("""			<div class="matrixec_vis3">%s</div>""" % vertical_title.replace("PLACEHOLDER", "Expected number of parts"))
			out.append("""</th>""")
			left_col_once=0
		if goal==0:
			out.append("""		<th align="center" class="acid_tab_skip">deleted</th>""")
		else:
			out.append("""		<th align="center" class="acid_tab_skip" style="min-width:50px;">""")
			out.append("""			<span class="matrixec_vis0">RL&#x202F;=&#x202F;%u</span>""" % (goal-1))
			out.append("""			<span class="matrixec_vis1">%s</span>""" % ("-" if (goal>9) else ("1&#x202F;+%u&#x202F;copies" % (goal-1))))
			out.append("""			<span class="matrixec_vis2">%s</span>""" % ("-" if (goal==1) else ("8&#x202F;+%u&#x202F;parts" % (goal-1))))
			if len(matrix)>=8:
				out.append("""			<span class="matrixec_vis3">%s</span>""" % ("-" if (goal==1) else ("4&#x202F;+%u&#x202F;parts" % (goal-1))))
			out.append("""		</th>""")
		# columns
		for actual in range(11):
			(col,clz)=redundancy2colclass(goal,actual)
			for i in range(len(matrix)):
				classsum[i][col]+=matrix[i][goal][actual]
			out.append("""		<td align="right" class="acid_tab_skip">""")
			if len(matrix)==8:
				out.append("""			<span class="matrixar_vis0">""")
				if matrix[0][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(matrix[0][goal][actual])))
				if matrix[2][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(matrix[2][goal][actual])))
				if matrix[4][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(matrix[4][goal][actual])))
				if matrix[6][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis3">%s</span>""" % (clz,decimal_number_html(matrix[6][goal][actual])))
				out.append("""			</span>""")
				out.append("""			<span class="matrixar_vis1">""")
				if matrix[1][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(matrix[1][goal][actual])))
				if matrix[3][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(matrix[3][goal][actual])))
				if matrix[5][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(matrix[5][goal][actual])))
				if matrix[7][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis3">%s</span>""" % (clz,decimal_number_html(matrix[7][goal][actual])))
				out.append("""			</span>""")
			elif len(matrix)==6:
				out.append("""			<span class="matrixar_vis0">""")
				if matrix[0][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(matrix[0][goal][actual])))
				if matrix[2][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(matrix[2][goal][actual])))
				if matrix[4][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(matrix[4][goal][actual])))
				out.append("""			</span>""")
				out.append("""			<span class="matrixar_vis1">""")
				if matrix[1][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(matrix[1][goal][actual])))
				if matrix[3][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(matrix[3][goal][actual])))
				if matrix[5][goal][actual]>0:
					out.append("""			<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(matrix[5][goal][actual])))
				out.append("""			</span>""")
			else:
				if matrix[0][goal][actual]>0:
					out.append("""			<span class="%s matrixar_vis0">%s</span>""" % (clz,decimal_number_html(matrix[0][goal][actual])))
				if matrix[1][goal][actual]>0:
					out.append("""			<span class="%s matrixar_vis1">%s</span>""" % (clz,decimal_number_html(matrix[1][goal][actual])))
			out.append("""		</td>""")

		if goal==0:
			clz="IGNORE"
		else:
			clz=""
		out.append("""		<td align="right" class="acid_tab_skip">""")
		if len(matrix)==8:
			out.append("""			<span class="matrixar_vis0">""")
			out.append("""				<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(sum(matrix[0][goal]))))
			out.append("""				<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(sum(matrix[2][goal]))))
			out.append("""				<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(sum(matrix[4][goal]))))
			out.append("""				<span class="%s matrixec_vis3">%s</span>""" % (clz,decimal_number_html(sum(matrix[6][goal]))))
			out.append("""			</span>""")
			out.append("""			<span class="matrixar_vis1">""")
			out.append("""				<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(sum(matrix[1][goal]))))
			out.append("""				<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(sum(matrix[3][goal]))))
			out.append("""				<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(sum(matrix[5][goal]))))
			out.append("""				<span class="%s matrixec_vis3">%s</span>""" % (clz,decimal_number_html(sum(matrix[7][goal]))))
			out.append("""			</span>""")
		elif len(matrix)==6:
			out.append("""			<span class="matrixar_vis0">""")
			out.append("""				<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(sum(matrix[0][goal]))))
			out.append("""				<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(sum(matrix[2][goal]))))
			out.append("""				<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(sum(matrix[4][goal]))))
			out.append("""			</span>""")
			out.append("""			<span class="matrixar_vis1">""")
			out.append("""				<span class="%s matrixec_vis0">%s</span>""" % (clz,decimal_number_html(sum(matrix[1][goal]))))
			out.append("""				<span class="%s matrixec_vis1">%s</span>""" % (clz,decimal_number_html(sum(matrix[3][goal]))))
			out.append("""				<span class="%s matrixec_vis2">%s</span>""" % (clz,decimal_number_html(sum(matrix[5][goal]))))
			out.append("""			</span>""")
		else:
			out.append("""			<span class="%s matrixar_vis0">%s</span>""" % (clz,decimal_number_html(sum(matrix[0][goal]))))
			out.append("""			<span class="%s matrixar_vis1">%s</span>""" % (clz,decimal_number_html(sum(matrix[1][goal]))))
		out.append("""		</td>""")
		out.append("""	</tr>""")

		if goal>0:
			for i in range(len(matrix)):
				sumlist[i] = [ a + b for (a,b) in zip(sumlist[i],matrix[i][goal])]

	out.append("""	<tr>""")
	out.append("""		<th align="center" class="acid_tab_skip">all 1+</th>""")
	for actual in range(11):
		out.append("""		<td align="right" class="acid_tab_skip">""")
		if len(matrix)==8:
			out.append("""			<span class="matrixar_vis0">""")
			out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sumlist[0][actual]))
			out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sumlist[2][actual]))
			out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sumlist[4][actual]))
			out.append("""				<span class="matrixec_vis3">%s</span>""" % decimal_number_html(sumlist[6][actual]))
			out.append("""			</span>""")
			out.append("""			<span class="matrixar_vis1">""")
			out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sumlist[1][actual]))
			out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sumlist[3][actual]))
			out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sumlist[5][actual]))
			out.append("""				<span class="matrixec_vis3">%s</span>""" % decimal_number_html(sumlist[7][actual]))
			out.append("""			</span>""")
		elif len(matrix)==6:
			out.append("""			<span class="matrixar_vis0">""")
			out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sumlist[0][actual]))
			out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sumlist[2][actual]))
			out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sumlist[4][actual]))
			out.append("""			</span>""")
			out.append("""			<span class="matrixar_vis1">""")
			out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sumlist[1][actual]))
			out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sumlist[3][actual]))
			out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sumlist[5][actual]))
			out.append("""			</span>""")
		else:
			out.append("""			<span class="matrixar_vis0">%s</span>""" % decimal_number_html(sumlist[0][actual]))
			out.append("""			<span class="matrixar_vis1">%s</span>""" % decimal_number_html(sumlist[1][actual]))
		out.append("""		</td>""")
	out.append("""		<td align="right" class="acid_tab_skip">""")
	if len(matrix)==8:
		out.append("""			<span class="matrixar_vis0">""")
		out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sum(sumlist[0])))
		out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sum(sumlist[2])))
		out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sum(sumlist[4])))
		out.append("""				<span class="matrixec_vis3">%s</span>""" % decimal_number_html(sum(sumlist[6])))
		out.append("""			</span>""")
		out.append("""			<span class="matrixar_vis1">""")
		out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sum(sumlist[1])))
		out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sum(sumlist[3])))
		out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sum(sumlist[5])))
		out.append("""				<span class="matrixec_vis3">%s</span>""" % decimal_number_html(sum(sumlist[7])))
		out.append("""			</span>""")
	elif len(matrix)==6:
		out.append("""			<span class="matrixar_vis0">""")
		out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sum(sumlist[0])))
		out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sum(sumlist[2])))
		out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sum(sumlist[4])))
		out.append("""			</span>""")
		out.append("""			<span class="matrixar_vis1">""")
		out.append("""				<span class="matrixec_vis0">%s</span>""" % decimal_number_html(sum(sumlist[1])))
		out.append("""				<span class="matrixec_vis1">%s</span>""" % decimal_number_html(sum(sumlist[3])))
		out.append("""				<span class="matrixec_vis2">%s</span>""" % decimal_number_html(sum(sumlist[5])))
		out.append("""			</span>""")
	else:
		out.append("""			<span class="matrixar_vis0">%s</span>""" % decimal_number_html(sum(sumlist[0])))
		out.append("""			<span class="matrixar_vis1">%s</span>""" % decimal_number_html(sum(sumlist[1])))
	out.append("""		</td>""")
	out.append("""	</tr>""")
	out.append("""	<tr><th align="center" class="acid_tab_skip"></th><td colspan="13" class="acid_tab_skip" style="padding-left:80px;padding-right:80px;">""")
	if len(matrix)==8:
		out.append("""		<span class="matrixar_vis0">""")
		out.append("""			<span class="matrixec_vis0">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[0][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis1">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[2][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis2">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[4][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis3">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[6][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""		</span>""")
		out.append("""		<span class="matrixar_vis1">""")
		out.append("""			<span class="matrixec_vis0">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[1][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis1">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[3][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis2">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[5][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis3">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[7][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""		</span>""")
	elif len(matrix)==6:
		out.append("""		<span class="matrixar_vis0">""")
		out.append("""			<span class="matrixec_vis0">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[0][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis1">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[2][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis2">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[4][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""		</span>""")
		out.append("""		<span class="matrixar_vis1">""")
		out.append("""			<span class="matrixec_vis0">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[1][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis1">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[3][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""			<span class="matrixec_vis2">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[5][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""		</span>""")
	else:
		out.append("""		<span class="matrixar_vis0">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[0][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
		out.append("""		<span class="matrixar_vis1">"""+("&nbsp;&nbsp;&nbsp;&nbsp;".join(["""<span class="%sBOX"></span>&nbsp;%s <span class="%s">%s</span>""" % (cl,desc,cl,decimal_number_html(classsum[1][clidx])) for clidx,cl,desc in [(0,"MISSING","missing"),(1,"ENDANGERED","endangered"),(2,"UNDERGOAL","undergoal"),(3,"NORMAL","stable"),(4,"OVERGOAL","overgoal"),(5,"DELETEPENDING","pending&nbsp;deletion"),(6,"DELETEREADY","ready&nbsp;to&nbsp;be&nbsp;removed")]]))+"</span>")
	out.append("""	</td></tr>""")
	out.append("""</table></form>""")
	return out


from common.utilsgui import *

def render(dp, fields, vld):
	# mu.memlabels,mu.abrlabels,mu.totalused,mu.totalallocated,mu.memusage = dp.get_memory_usage()
	mu = dp.get_memory_usage()
	out = []
	out.append("""<div class="tab_title">Memory usage details</div>""")
	out.append("""<table class="FR">""")
	out.append("""	<tr><th></th>""")
	for label in mu.memlabels:
		out.append("""		<th style="min-width: 70px;">%s</th>""" % label)
	out.append("""	<th style="min-width: 70px;">total</th></tr>""")
	out.append("""	<tr><th align="center">Used</th>""")
	for i in range(len(mu.memlabels)):
		out.append("""		<td align="center"><a style="cursor:default" title="%s B">%s</a></td>""" % (decimal_number(mu.memusage[1+i*2]),humanize_number(mu.memusage[1+i*2],"&nbsp;")))
	out.append("""	<td align="center"><a style="cursor:default" title="%s B">%s</a></td></tr>""" % (decimal_number(mu.totalused),humanize_number(mu.totalused,"&nbsp;")))
	out.append("""	<tr><th align="center">Allocated</th>""")
	for i in range(len(mu.memlabels)):
		out.append("""		<td align="center"><a style="cursor:default" title="%s B">%s</a></td>""" % (decimal_number(mu.memusage[i*2]),humanize_number(mu.memusage[i*2],"&nbsp;")))
	out.append("""	<td align="center"><a style="cursor:default" title="%s B">%s</a></td></tr>""" % (decimal_number(mu.totalallocated),humanize_number(mu.totalallocated,"&nbsp;")))
	out.append("""	<tr><th align="center">Utilization</th>""")
	for i in range(len(mu.memlabels)):
		if mu.memusage[i*2]:
			percent = "%.2f %%" % (100.0 * mu.memusage[1+i*2] / mu.memusage[i*2])
		else:
			percent = "-"
		out.append("""		<td align="center">%s</td>""" % percent)
	if mu.totalallocated:
		percent = "%.2f %%" % (100.0 * mu.totalused / mu.totalallocated)
	else:
		percent = "-"
	out.append("""	<td align="center">%s</td></tr>""" % percent)
	if mu.totalallocated>0:
		out.append("""	<tr><th rowspan="2" align="center">Distribution</th>""")
		for i in range(len(mu.memlabels)):
			tpercent = "%.2f %%" % (100.0 * mu.memusage[i*2] / mu.totalallocated)
			out.append("""		<td align="center">%s</td>""" % tpercent)
		out.append("""	<td>-</td></tr>""")
		out.append("""  <tr>""")
		out.append("""		<td colspan="%d" class="NOPADDING">""" % (len(mu.memlabels)+1))
		out.append("""			<table width="100%" style="border:0px;" id="bar"><tr>""")
		memdistribution = []
		other = 0.0
		for i,(label,abr) in enumerate(zip(mu.memlabels,mu.abrlabels)):
			tpercent = (100.0 * mu.memusage[i*2] / mu.totalallocated)
			if tpercent>5.0:
				memdistribution.append((tpercent,label,abr))
			else:
				other+=tpercent
		memdistribution.sort()
		memdistribution.reverse()
		if other>0:
			memdistribution.append((other,None,None))
		clz = "FIRST"
		labels = []
		tooltips = []
		for i,(percent,label,abr) in enumerate(memdistribution):
			if label:
				if percent>7.0:
					out.append("""				<td style="width:%.2f%%;" class="MEMDIST%d MEMDIST%s" align="center"><a style="cursor:default;" title="%s (%.2f %%)">%s</a></td>""" % (percent,i,clz,label,percent,label))
				elif percent>3.0:
					out.append("""				<td style="width:%.2f%%;" class="MEMDIST%d MEMDIST%s" align="center"><a style="cursor:default;" title="%s (%.2f %%)">%s</a></td>""" % (percent,i,clz,label,percent,abr)) 
				else:
					out.append("""				<td style="width:%.2f%%;" class="MEMDIST%d MEMDIST%s" align="center"><a style="cursor:default;" title="%s (%.2f %%)">%s</a></td>""" % (percent,i,clz,label,percent,"#")) 
				labels.append(label)
				tooltips.append("%s (%.2f %%)" % (label,percent))
			else:
				out.append("""				<td style="width:%.2f%%;" class="MEMDISTOTHER MEMDIST%s">others</td>""" % (percent,clz))
				labels.append("others")
				tooltips.append("other memory segments (%.2f %%)" % (percent))
			clz = "MID"
		out.append("""			</tr></table>""")
		out.append("""<script type="text/javascript">""")
		out.append("""<!--//--><![CDATA[//><!--""")
		out.append("""	var bar_labels = [%s];""" % ",".join(map(repr,labels)))
		out.append("""	var bar_tooltips = [%s];""" % ",".join(map(repr,tooltips)))
		out.append("""//--><!]]>""")
		out.append("""</script>""")
		out.append("""<script type="text/javascript">
<!--//--><![CDATA[//><!--
function bar_refresh() {
var b = document.getElementById("bar");
var i,j,x;
if (b) {
var x = b.getElementsByTagName("td");
for (i=0 ; i<x.length ; i++) {
x[i].innerHTML = "";
}
for (i=0 ; i<x.length ; i++) {
var width = x[i].clientWidth;
var label = bar_labels[i];
var tooltip = bar_tooltips[i];
x[i].innerHTML = "<a title='" + tooltip + "'>" + label + "</a>";
if (width<x[i].clientWidth) {
	x[i].innerHTML = "<a title='" + tooltip + "'>&#8230;</a>";
	if (width<x[i].clientWidth) {
		x[i].innerHTML = "<a title='" + tooltip + "'>&#8226;</a>";
		if (width<x[i].clientWidth) {
			x[i].innerHTML = "<a title='" + tooltip + "'>.</a>";
			if (width<x[i].clientWidth) {
				x[i].innerHTML = "";
			}
		}
	} else {
		for (j=1 ; j<bar_labels[i].length-1 ; j++) {
			x[i].innerHTML = "<a title='" + tooltip + "'>"+label.substring(0,j) + "&#8230;</a>";
			if (width<x[i].clientWidth) {
				break;
			}
		}
		x[i].innerHTML = "<a title='" + tooltip + "'>" + label.substring(0,j-1) + "&#8230;</a>";
	}
}
}
}
}

function bar_add_event(obj,type,fn) {
if (obj.addEventListener) {
obj.addEventListener(type, fn, false);
} else if (obj.attachEvent) {
obj.attachEvent('on'+type, fn);
}
}

//bar_add_event(window,"load",bar_refresh); - comment due to flickering on load (for short bars <10%)
bar_add_event(window,"resize",bar_refresh);
//--><!]]>
</script>""")
		out.append("""		</td>""")
		out.append("""	</tr>""")
	out.append("""</table>""")
	return out


import socket 
import struct
import time
from datetime import datetime

from common.constants import *
from common.errors import *

UNIQ_MASK_IP = 1 << (1+ord('Z')-ord('A'))
UNIQ_MASK_RACK = 1 << (2+ord('Z')-ord('A'))

def myunicode(x):
	return str(x)

def gettzoff(t):
	return time.localtime(t).tm_gmtoff

def shiftts(t):
	return t - gettzoff(t)

def safe_int(val, default=0):
	try:
		return int(val)
	except Exception:
		return default

def resolve(strip, donotresolve=0, default=None):
	if donotresolve:
		return strip
	try:
		return (socket.gethostbyaddr(strip))[0]
	except Exception:
		if default is not None:
			return default
		else:
			return strip

def decimal_number(number,sep=' '):
	parts = []
	while number>=1000:
		number,rest = divmod(number,1000)
		parts.append("%03u" % rest)
	parts.append(str(number))
	parts.reverse()
	return sep.join(parts)

def decimal_number_html(number):
	return decimal_number(number,"&#8239;")

# returns decimal number with non-breaking space as separator or 'n/a' if number is None
def decimal_number_na_html(number):
	if number is None:
		return 'n/a'
	return decimal_number(number,"&#8239;")


def humanize_number(number,sep='',suff='B'):
	number*=100
	scale=0
	while number>=99950:
		number = number//1024
		scale+=1
	if number<995 and scale>0:
		b = (number+5)//10
		nstr = "%u.%u" % divmod(b,10)
	else:
		b = (number+50)//100
		nstr = "%u" % b
	if scale>0:
		return "%s%s%si%s" % (nstr,sep,"-KMGTPEZY"[scale],suff)
	else:
		return "%s%s%s" % (nstr,sep,suff)

# return ordinal suffix for a number
def ordinal_suffix(i):
	if i%100 in (11,12,13):
		return "th"
	elif i%10==1:
		return "st"
	elif i%10==2:
		return "nd"
	elif i%10==3:
		return "rd"
	else:
		return "th"

def timeduration_to_shortstr(timeduration, sep=''):
	for l,s in ((604800,'w'),(86400,'d'),(3600,'h'),(60,'m'),(0,'s')):
		if timeduration>=l:
			if l>0:
				n = float(timeduration)/float(l)
			else:
				n = float(timeduration)
			rn = round(n,1)
			if n==round(n,0):
				return "%.0f%s%s" % (n,sep,s)
			else:
				return "%s%.1f%s%s" % (("~"+sep if n!=rn else ""),rn,sep,s)
	return "???"

def timeduration_to_fullstr(timeduration):
	if timeduration>=86400:
		days,dayseconds = divmod(timeduration,86400)
		daysstr = "%u day%s + " % (days,("s" if days!=1 else ""))
	else:
		dayseconds = timeduration
		daysstr = ""
	hours,hourseconds = divmod(dayseconds,3600)
	minutes,seconds = divmod(hourseconds,60)
	if seconds==round(seconds,0):
		return "%u second%s (%s%u:%02u:%02u)" % (timeduration,("" if timeduration==1 else "s"),daysstr,hours,minutes,seconds)
	else:
		seconds,fracsec = divmod(seconds,1)
		return "%.3f seconds (%s%u:%02u:%02u.%03u)" % (timeduration,daysstr,hours,minutes,seconds,round(1000*fracsec,0))

def hours_to_str(hours, sep=''):
	days,hoursinday = divmod(hours,24)
	if days>0 and hours!=24:
		if hoursinday>0:
			return "%u%sd %u%sh" % (days,sep,hoursinday,sep)
		else:
			return "%u%sd" % (days,sep)
	else:
		return "%u%sh" % (hours,sep)

def datetime_to_str(tm):
	return time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(tm))

def date_to_str(tm):
	return time.strftime("%Y-%m-%d",time.localtime(tm))

def time_to_str(tm):
	return time.strftime("%H:%M:%S",time.localtime(tm))

def seconds_since_beginning_of_hour():
	now = datetime.now()  # Get the current date and time
	start_of_hour = now.replace(minute=0, second=0, microsecond=0)  # Set minutes, seconds, and microseconds to 0
	seconds_since = (now - start_of_hour).total_seconds()  # Calculate the difference in seconds
	return int(seconds_since)  # Return the result as an integer

def label_id_to_char(id):
	return chr(ord('A')+id)

def state_name(stateid):
	if   stateid==STATE_DUMMY:    return "DUMMY"
	elif stateid==STATE_USURPER:  return "USURPER"
	elif stateid==STATE_FOLLOWER: return "FOLLOWER"
	elif stateid==STATE_ELECT:    return "ELECT"
	elif stateid==STATE_DEPUTY:   return "DEPUTY"
	elif stateid==STATE_LEADER:   return "LEADER"
	elif stateid==STATE_MASTERCE: return "MASTER"
	else:                         return "???"

def state_color(stateid,sync):
	if stateid==STATE_DUMMY:
		return 8
	elif stateid==STATE_FOLLOWER or stateid==STATE_USURPER:
		if sync:
			return 5
		else:
			return 6
	elif stateid==STATE_ELECT:
		return 3
	elif stateid==STATE_DEPUTY:
		return 2
	elif stateid==STATE_LEADER:
		return 4
	else:
		return 1

def labelmask_to_str(labelmask):
	str = ""
	m = 1
	for i in range(26):
		if labelmask & m:
			str += label_id_to_char(i)
		m <<= 1
	return str

def labelmasks_to_str(labelmasks):
	if labelmasks[0]==0:
		return "*"
	r = []
	for labelmask in labelmasks:
		if labelmask==0:
			break
		r.append(labelmask_to_str(labelmask))
	return "+".join(r)

def labelexpr_to_str(labelexpr, literals = ['*','&','|','~']):
	any = literals[0]
	andop = literals[1]
	orop = literals[2]
	notop = literals[3]
	stack = []
	if len(labelexpr)==0:
		return any
	if labelexpr[0]==0:
		return any              # '*' any label
	for i in labelexpr:
		if i==255:
			stack.append((0,any)) # '*' any label
		elif i>=192 and i<255:
			n = (i-192)
			stack.append((0,chr(ord('A')+n)))
		elif i>=128 and i<192:
			n = (i-128)+2
			if n>len(stack):
				return 'EXPR ERROR'
			m = []
			for _ in range(n):
				l,s = stack.pop()
				if l>1:
					m.append("(%s)" % s)
				else:
					m.append(s)
			m.reverse()
			stack.append((1,andop.join(m)))   # '&' and operator
		elif i>=64 and i<128:
			n = (i-64)+2
			if n>len(stack):
				return 'EXPR ERROR'
			m = []
			for _ in range(n):
				l,s = stack.pop()
				if l>2:
					m.append("(%s)" % s)
				else:
					m.append(s)
			m.reverse()
			stack.append((2,orop.join(m)))   # '|' or operator
		elif i==1:
			if len(stack)==0:
				return 'EXPR ERROR'
			l,s = stack.pop()
			if l>0:
				stack.append((0,notop+"(%s)" % s)) # '~' not operator
			else:
				stack.append((0,notop+"%s" % s))   # '~' not operator
		elif i==0:
			break
		else:
			return 'EXPR ERROR'
	if len(stack)!=1:
		return 'EXPR ERROR'
	l,s = stack.pop()
	return s

def labellist_fold(labellist):
	ll = []
	prev_data = None
	count = 0
	for data in labellist:
		if (data != prev_data):
			if count>0:
				if count>1:
					ll.append(('%u%s' % (count,prev_data[0]),prev_data[1],prev_data[2]))
				else:
					ll.append(prev_data)
			prev_data = data
			count = 1
		else:
			count = count+1
	if count>0:
		if count>1:
			ll.append(('%u%s' % (count,prev_data[0]),prev_data[1],prev_data[2]))
		else:
			ll.append(prev_data)
	return ll

def uniqmask_to_str(uniqmask):
	if uniqmask==0:
		return "-"
	if uniqmask & UNIQ_MASK_IP:
		return "[IP]"
	if uniqmask & UNIQ_MASK_RACK:
		return "[RACK]"
	rstr = ""
	inrange = 0
	for i in range(26):
		if uniqmask & (1<<i):
			if inrange==0:
				rstr += chr(ord('A')+i)
				if i<24 and ((uniqmask>>i)&7)==7:
					inrange = 1
		else:
			if inrange==1:
				rstr += '-'
				rstr += chr(ord('A')+(i-1))
				inrange = 0
	if inrange:
		rstr += '-'
		rstr += chr(ord('A')+25)
	return rstr

def eattr_to_str(seteattr,clreattr):
	eattrs = ["noowner","noattrcache","noentrycache","nodatacache","snapshot","undeletable","appendonly","immutable"]
	outlist = []
	mask = 1
	for eattrname in eattrs:
		if seteattr&mask:
			outlist.append("+%s" % eattrname)
		if clreattr&mask:
			outlist.append("-%s" % eattrname)
		mask = mask << 1
	if len(outlist)>0:
		return ",".join(outlist)
	else:
		return "-"

def get_string_from_packet(data,pos,err):
	rstr = ""
	shift = 0
	if 1+pos > len(data):
		err = 1;
	if err==0:
		shift = data[pos]
		if shift+1+pos > len(data):
			err = 1
		else:
			rstr = data[pos+1:pos+1+shift]
		rstr = rstr.decode('utf-8','replace')
	return (rstr,pos+shift+1,err)

def get_longstring_from_packet(data,pos,err):
	rstr = ""
	shift = 0
	if 2+pos > len(data):
		err = 1;
	if err==0:
		shift = data[pos]*256+data[pos+1]
		if shift+2+pos > len(data):
			err = 1
		else:
			rstr = data[pos+2:pos+2+shift]
		rstr = rstr.decode('utf-8','replace')
	return (rstr,pos+shift+2,err)

def version_convert(versionxyzp):
	if versionxyzp>=(4,0,0) and versionxyzp<(4,11,0):
		return (versionxyzp,0)
	elif versionxyzp>=(2,0,0):
		return ((versionxyzp[0],versionxyzp[1],versionxyzp[2]//2),versionxyzp[2]&1)
	elif versionxyzp>=(1,7,0):
		return (versionxyzp,1)
	elif versionxyzp>(0,0,0):
		return (versionxyzp,0)
	else:
		return (versionxyzp,-1)

def version_str_sort_pro(versionxyzp):
	versionxyz,pro = version_convert(versionxyzp)
	strver = "%u.%u.%u" % versionxyz
	sortver = "%05u_%03u_%03u" % versionxyz
	if pro==1:
		strver += " PRO"
		sortver += "_2"
	elif pro==0:
		sortver += "_1"
	else:
		sortver += "_0"
	if strver == '0.0.0':
		strver = ''
	return (strver,sortver,pro)

# converts redundancy goal and actual to class and column number
# 0: missing, 1: endangered, 2: undergoal, 3: normal, 4: overgoal, 5: delete pending, 6: delete ready
def redundancy2colclass(goal,actual):
	if goal==0:
		if actual==0:
			clz = "DELETEREADY"
			col = MX_COL_DELETEREADY
		else:
			clz = "DELETEPENDING"
			col = MX_COL_DELETEPENDING
	elif actual==0:
		clz = "MISSING"
		col = MX_COL_MISSING
	elif actual>goal:
		clz = "OVERGOAL"
		col = MX_COL_OVERGOAL
	elif actual<goal:
		if actual==1:
			clz = "ENDANGERED"
			col = MX_COL_ENDANGERED
		else:
			clz = "UNDERGOAL"
			col = MX_COL_UNDERGOAL
	else:
		clz = "NORMAL"
		col = MX_COL_STABLE
	return (col,clz)

def disablesmask_to_string_list(disables_mask):
	cmds = ["chown","chmod","symlink","mkfifo","mkdev","mksock","mkdir","unlink","rmdir","rename","move","link","create","readdir","read","write","truncate","setlength","appendchunks","snapshot","settrash","setsclass","seteattr","setxattr","setfacl"]
	l = []
	m = 1
	for cmd in cmds:
		if disables_mask & m:
			l.append(cmd)
		m <<= 1
	return l

def disablesmask_to_string(disables_mask):
	return ",".join(disablesmask_to_string_list(disables_mask))




def charts_convert_data(datalist,mul,div,raw):
	res = []
	nodata = (2**64)-1
	for v in datalist:
		if v==nodata:
			res.append(None)
		else:
			if raw:
				res.append(v)
			else:
				res.append((v*mul)/div)
	return res

# Unpacks chart data from a given (ANTOCL_CHART_DATA) packet
def unpack_chart_packet(data,length):
	if length>=8:
		ranges,series,entries,perc,base = struct.unpack(">BBLBB",data[:8])
		if length==8+ranges*(13+series*entries*8):
			res = {}
			unpackstr = ">%uQ" % entries
			for r in range(ranges):
				rpos = 8 + r * (13+series*entries*8)
				rng,ts,mul,div = struct.unpack(">BLLL",data[rpos:rpos+13])
				rpos += 13
				if series>3:
					series=3
				l1 = None
				l2 = None
				l3 = None
				if series>=1:
					l1 = list(struct.unpack(unpackstr,data[rpos:rpos+entries*8]))
				if series>=2:
					l2 = list(struct.unpack(unpackstr,data[rpos+entries*8:rpos+2*entries*8]))
				if series>=3:
					l3 = list(struct.unpack(unpackstr,data[rpos+2*entries*8:rpos+3*entries*8]))
				res[rng] = (l1,l2,l3,ts,mul,div)
			return perc,base,res
		else:
			return None,None,None
	else:
		return None,None,None

# Handles a single chart data request (CLTOAN_CHART_DATA) to the given server (mfsconn)
def get_charts_multi_data(mfsconn,chartid,dataleng):
	data,length = mfsconn.command(CLTOAN_CHART_DATA,ANTOCL_CHART_DATA,struct.pack(">LLB",chartid,dataleng,1))
	return unpack_chart_packet(data,length)

# Handles multiple, concurrent chart data requests (CLTOAN_CHART_DATA) to many chunkservers at once
def get_charts_multi_data_async(mfsmulticonn,chartid_arr,dataleng):
	all_charts = {}
	dataout_arr = []
	for chartid in chartid_arr:
		dataout_arr.append(struct.pack(">LLB",chartid,dataleng,1))
	answers=mfsmulticonn.command(CLTOAN_CHART_DATA,ANTOCL_CHART_DATA,dataout_arr) #ask all registered chunkservers about chart data
	for hostkey in answers: # hostkey is string: 'ip:port'
		data_arr,length_arr = answers[hostkey]
		if len(chartid_arr)!=len(data_arr) or len(chartid_arr)!=len(length_arr):
			raise MFSCommunicationError("charts data length mismatch")
		for i in range(len(chartid_arr)):
			chartid = chartid_arr[i]
			data = data_arr[i]
			length = length_arr[i]
			perc,base,res = unpack_chart_packet(data,length)
			if perc!=None and base!=None and res!=None:
				chartkey = hostkey + ":" + str(chartid)
				all_charts[chartkey] = (perc,base,res)
	return all_charts

# Adjust chart data multiplication/division factors based on the base value
def adjust_muldiv(mul,div,base):
	base -= 2
	if base>0:
		mul = mul*(1000**base)
	if base<0:
		div = div*(1000**(-base))
	return mul,div



import os
import math
import traceback

nnbsp = "&#8239;" # narrow non-breaking space

from common.utils import *

class MFSFieldStorage:
	def __init__(self):
		self.data = {}
	def __repr__(self):
		return repr(self.data)
	def __str__(self):
		return str(self.data)
	def __contains__(self,key):
		return key in self.data
	def __iter__(self):
		return iter(self.data.keys())
	def initFromURL(self):
		for k,v in xurllib.parse_qsl(os.environ["QUERY_STRING"]):
			if k in self.data:
				if type(self.data[k])==str:
					x = self.data[k]
					self.data[k] = [x]
				self.data[k].append(v)
			else:
				self.data[k] = v
	def getvalue(self,key,default=None):
		if key in self.data:
			if type(self.data[key]) is list and len(self.data[key])>=1:
				return self.data[key][0]
			return self.data[key]
		return default
	def getint(self,key,default=None):
		try:
			return int(self.getvalue(key,default))
		except Exception:
			return default
	def getstr(self,key,default=None):
		return str(self.getvalue(key,default))
	def pop(self,key):
		return self.data.pop(key)
	def append(self, key, value):
		self.data[key]=value

	def createhtmllink(self, update):
		fields = self
		c = []
		for k in fields:
			if k not in update:
				f = fields.getvalue(k)
				if type(f) is list:
					for el in f:
						c.append("%s=%s" % (k,urlescape(el)))
				elif type(f) is str:
					c.append("%s=%s" % (k,urlescape(f)))
		for k,v in update.items():
			if v!="":
				c.append("%s=%s" % (k,urlescape(v)))
		return "mfs.cgi?%s" % ("&amp;".join(c))

	def createrawlink(self, update):
		fields = self
		c = []
		for k in fields:
			if k not in update:
				f = fields.getvalue(k)
				if type(f) is list:
					for el in f:
						c.append("%s=%s" % (k,urlescape(el)))
				elif type(f) is str:
					c.append("%s=%s" % (k,urlescape(f)))
		for k,v in update.items():
			if v!="":
				c.append("%s=%s" % (k,urlescape(v)))
		return ("mfs.cgi?%s" % ("&".join(c))).replace('"','').replace("'","")

	def createorderlink(self, prefix,columnid):
		fields = self
		ordername = "%sorder" % prefix
		revname = "%srev" % prefix
		orderval = fields.getint(ordername,0)
		revval = fields.getint(revname,0)
		return self.createhtmllink({revname:"1"}) if orderval==columnid and revval==0 else self.createhtmllink({ordername:str(columnid),revname:"0"})

	def createinputs(self, ignorefields):
		fields = self
		for k in fields:
			if k not in ignorefields:
				f = fields.getvalue(k)
				if type(f) is list:
					for el in f:
						yield """<input type="hidden" name="%s" value="%s">""" % (k,htmlentities(el))
				elif type(f) is str:
					yield """<input type="hidden" name="%s" value="%s">""" % (k,htmlentities(f))
		return
	

try:
	import urllib.parse as xurllib
except ImportError:
	import urllib as xurllib

def htmlentities(str):
	try:
		return str.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;').replace("'",'&apos;').replace('"','&quot;')
	except:
		return ""
	
def urlescape(str):
	return xurllib.quote_plus(str)



# Generates a simple vertical table with a title row (if provided) and individual td styling (if provided)
def html_table_vertical(title, tdata, tablecls=""):
	out=[]
	out.append("""<table class="FR vertical no-hover %s">""" % tablecls)
	if title:
		out.append(""" <thead><tr><th colspan="2">%s</th></tr></thead>""" % title)
	out.append(""" <tbody>""")
	for row in tdata:
		out.append("""		<tr><th>%s</th>""" % str(row[0]))
		out.append("""		<td class="%s">%s</td></tr>""" % ("center" if len(row)<3 else row[2], str(row[1])))
	out.append(""" </tbody>""")
	out.append("""</table>""")
	return "\n".join(out)

# Generates a SVG-based round knob for selecting (from 2 to 6) different options
# id - string knob unique identifier
# r - circle radius
# wh - tuple (width, height) of svg picture
# cxy - tuple (cx,cy) center of knob circle 
# opts - a list of options, each option is a tuple: (degrees (0=north), option_title, option_subtitle, onclick_function)
# store - flag if knob position should be stored in the browser session storage
# selected - selected option index
def html_knob_selector(id, r, wh, cxy, opts, store=True, selected=None):
	w=wh[0]
	h=wh[1]
	cx=cxy[0]
	cy=cxy[1]	
	rx = r+4  #x-offset from circle center to line break
	tx = 12   #x-text offset from line break
	store_cls = "" if store else "dont-store"

	degrees_arr = []
	fun_arr = []
	out =[]

	# prepare onclick-s for all options
	for i in range(len(opts)): #display titles, subtitles
		opt=opts[i]
		degrees=opt[0]
		name=opt[1]
		subtitle=opt[2]
		if len(name)<4:
			name= name+"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" if degrees>0 else "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" + name
		anchor = "start" if degrees>0 else "end"
		x = cx+rx+tx if degrees>0 else cx-rx-tx #text x-pos
		y = cy + math.tan((90+abs(degrees)) * math.pi / 180)*rx + 3 #text y-pos
		y = max(y,10)
		#setup a function (w/timeout) to call on a one of knob's title click
		out.append("""<g onclick="rotateKnob('%s',%u);setTimeout(() => {%s;}, 300);"><text class="option-name" style="text-anchor: %s;" x="%f" y="%f">%s</text>""" % (id,degrees,opt[3],anchor,x,y,name))
		if subtitle!=None: #display option's subtitle if there is any
			out.append("""<text class="option-sub-name" style="text-anchor: %s;" x="%f" y="%f">%s</text>""" % (anchor,x,y+12,subtitle))
		out.append("""</g>""")
		degrees_arr.append(str(degrees))
		#setup a function (w/timeout) to call on a knob circle click (auto rotate) 
		fun_arr.append("setTimeout(() => {%s;}, 300);" % opt[3].replace("'", '&quot;')) 
	
	# knob's circle onclick rotation
	knob_onclick = """rotateNextKnob('""" +id+ """',["""
	knob_onclick += ",".join(degrees_arr)
	knob_onclick += """],[' """ 
	knob_onclick += "', '".join(fun_arr)
	knob_onclick += """ '])"""
	out.append("""<g transform="translate(%f,%f)" onclick="%s">""" % (cx, cy, knob_onclick))

	#draw lines
	for i in range(len(opts)):
		opt=opts[i]
		degrees=opt[0]
		anchor = "start" if degrees>0 else "end"
		x = rx if degrees>0 else -rx #line break x-pos
		x1 = x+8 if degrees>0 else x-8
		y = math.tan((90+abs(degrees)) * math.pi / 180)*rx #line break y-pos
		out.append("""<polyline class="line" points="0 0 %f %f %f %f"/>""" % (x,y,x1,y))		

	out.append("""<circle cx="0" cy="0" r="%f" class="circle-background" /><circle cx="0" cy="0" r="%f" class="circle-foreground" />""" % (r,r))
	if selected==None:
		initial_rot = 'data-initial-rotation="%u"' % opts[0][0]
	else:
		initial_rot = 'data-selected-rotation="%u"' % opts[selected][0]
	out.append("""<g id="%s" class="knob-arrow %s" style="transform: rotate(var(--%s-knob-rotation));" %s><polygon points="0 2 2 1 2 -%f 0 -%f -2 -%f -2 1" /></g>""" % (id,store_cls,id,initial_rot,r+1,r+3,r+1))
	out.append("""</g></svg>""")
	out.insert(0,"""<svg viewbox="0 0 %f %f" width="%f" height="%f" class="knob-selector" xmlns="http://www.w3.org/2000/svg">""" % (w,h,w,h))
	return "\n".join(out)

#Helper for generating server chart's time-selector knob
def html_knob_selector_chart_range(id):
	options=[(-135,"2 days",  None,"AcidChartSetRange('%s',0)" % id),
			(-45,"2 weeks", None,"AcidChartSetRange('%s',1)" % id),
			(45,  "3 months",None,"AcidChartSetRange('%s',2)" % id),
			(135, "10 years",None,"AcidChartSetRange('%s',3)" % id)]
	return html_knob_selector(id,9,(170,34),(85,16),options,False)

# Generates a full span+svg tag with an icon
def html_icon(id, help='', scale=1, clz=''):
	if help:
		return '<svg class="icon" height="12px" width="12px" data-tt="%s"><use class="%s" transform="scale(%f)" xlink:href="#%s"/></svg>' % (help,clz,scale,id)
	else:
		return '<svg class="icon" height="12px" width="12px"><use class="%s" transform="scale(%f)" xlink:href="#%s"/></svg>' % (clz,scale,id)

def html_refresh_a(content, param=None, new_param_value=None):
	if param and new_param_value:
		return """<a class="VISIBLELINK" href="javascript:void(0);" onclick="refreshWithNewParam('%s','%s');">%s</a>""" % (param, new_param_value, content)
	else:
		return """<a class="VISIBLELINK" href="javascript:void(0);" onclick="refreshWithNewParam();">%s</a>""" % (content)

# Prints provided 'out' table with lines of text
def print_out(out):
	print("\n".join(out))

def hours2html(hours):
	return hours_to_str(hours, nnbsp)


# common auxiliary functions
def html_menu_subsections(org, fields, section):
	print("""<div class="submenu">""")
	for k in org.menu_subitems(section):
		name=org.sections_defs[k][0]
		if k in org.subsectionset:
			active = 'active'
			if len(org.subsectionset)<=1:
				hrefname=None
				hreficon=None
			else:
				hrefname=fields.createhtmllink({"subsections":k})
				hreficon=fields.createhtmllink({"subsections":"-".join(org.subsectionset-set([k]))})
		else:
			active = ''
			hrefname=fields.createhtmllink({"subsections":k})
			hreficon=fields.createhtmllink({"subsections":"-".join(org.subsectionset|set([k]))})
		for responsive in ["xs md", "sm lg"]:
			if hrefname:
				print("""<div class="submenu-item %s %s"><a href="%s">%s</a><a href="%s"><svg class="thumbtack %s"><use xlink:href="#icon-thumbtack"/></svg></a></div>""" % (responsive, active,hrefname,name,hreficon,active))
			else:
				print("""<div class="submenu-item %s %s">%s<svg class="thumbtack %s"><use xlink:href="#icon-thumbtack"/></svg></div>""" % (responsive, active,name,active))
	print("""</div>""")

def print_error(msg):
	out = []
	out.append("""<div class="tab_title ERROR">Oops!</div>""")
	out.append("""<table class="FR MESSAGE">""")
	out.append("""	<tr><td align="left"><span class="ERROR">An error has occurred:</span> %s</td></tr>""" % msg)
	out.append("""</table>""")
	print("\n".join(out))

def print_exception():
	# exc_type, exc_value, exc_traceback = sys.exc_info()
	try:
		print("""<div class="tab_title ERROR">Oops!</div>""")
		print("""<table class="FR MESSAGE">""")
		print("""<tr><td align="left"><span class="ERROR">An error has occurred. Check your MooseFS configuration and network connections. </span><br/>If you decide to seek support because of this error, please include the following traceback:""")
		print("""<pre>""")
		print(traceback.format_exc().strip())
		print("""</pre></td></tr>""")
		print("""</table>""")
	except Exception:
		print(traceback.format_exc().strip())



from common.utils import *
from common.utilsgui import *

def render(dp, fields, vld):
	CCdata = fields.getstr("CCdata", "")

	out = []

	hostlist = []
	for cs in dp.get_chunkservers(): # default sort by ip/port
		if (cs.flags&1)==0:
			hostlist.append((cs.ip,cs.port,cs.version))
	icharts = (
		(100,'cpu','cpu usage (percent)','<b>cpu usage</b>, sys: BOXGR2A user: BOXGR2B','sys: BOXGR2A user: BOXGR2B'),
		(107,'memory','memory usage (if available) rss + virt','<b>memory usage</b> (if available), rss: BOXGR2A + virt: BOXGR2B','rss: BOXGR2A + virt: BOXGR2B'),
		(109,'space','raw disk space usage (used/total)','<b>raw disk space usage</b>, used: BOXGR2A total: BOXGR2B','used: BOXGR2A total: BOXGR2B'),
		(110,'chunks','number of stored chunks (ec8/ec4/copy)','<b>number of stored chunks</b>, ec8: BOXGR3A ec4: BOXGR3B copy: BOXGR3C','ec8: BOXGR3A ec4: BOXGR3B copy: BOXGR3C'),
		(111,'hddcnt','number of folders (hard drives) with data','<b>number of folders (hard drives) with data</b>, damaged: BOXGR3A marked for removal: BOXGR3B working: BOXGR3C','damaged: BOXGR3A marked for removal: BOXGR3B working: BOXGR3C'),
		(47,'udiff','usage difference between the most and least used disk','',''),
		(28,'load','max number operations waiting in queue','',''),
		(101,'datain','traffic from clients and other chunkservers (bits/s - main server + replicator)','<b>traffic from clients and other chunkservers</b> (bits/s), main server: BOXGR2A + replicator: BOXGR2B','main server: BOXGR2A + replicator: BOXGR2B'),
		(102,'dataout','traffic to clients and other chunkservers (bits/s - main server + replicator)','<b>traffic to clients and other chunkservers</b> (bits/s), main server: BOXGR2A + replicator: BOXGR2B','main server: BOXGR2A + replicator: BOXGR2B'),
		(103,'bytesr','bytes read - data/other (bytes/s)','<b>bytes read</b> (bytes/s), data: BOXGR2A other: BOXGR2B','data: BOXGR2A other: BOXGR2B'),
		(104,'bytesw','bytes written - data/other (bytes/s)','<b>bytes written</b> (bytes/s), data: BOXGR2A other: BOXGR2B','data: BOXGR2A other: BOXGR2B'),
		(2,'masterin','traffic from master (bits/s)','',''),
		(3,'masterout','traffic to master (bits/s)','',''),
		(105,'hddopr','number of low-level read operations (per minute)','<b>number of low-level read operations</b> (per minute), data: BOXGR2A other: BOXGR2B','data: BOXGR2A other: BOXGR2B'),
		(106,'hddopw','number of low-level write operations (per minute)','<b>number of low-level write operations</b> (per minute), data: BOXGR2A other: BOXGR2B','data: BOXGR2A other: BOXGR2B'),
		(16,'hlopr','number of high-level read operations (per minute)','',''),
		(17,'hlopw','number of high-level write operations (per minute)','',''),
		(18,'rtime','time of data read operations','',''),
		(19,'wtime','time of data write operations','',''),
		(20,'repl','number of chunk replications (per minute)','',''),
		(21,'create','number of chunk creations (per minute)','',''),
		(22,'delete','number of chunk deletions (per minute)','',''),
		(33,'change','number of chunk internal operations duplicate, truncate, etc. (per minute)','',''),
		(108,'move','number of chunk internal rebalances per minute (high speed + low speed)','<b>number of chunk internal rebalances</b> (per minute), high speed: BOXGR2A + low speed: BOXGR2B','high speed: BOXGR2A + low speed: BOXGR2B')
	)

	charts = []
	for id,oname,desc,fdesc,sdesc in icharts:
		if fdesc=='':
			fdesc = '<b>' + desc.replace(' (', '</b> (')
			if not '</b>' in fdesc:
				fdesc = fdesc + '</b>'
		fdeschtml = fdesc.replace('BOXGR1A','<span class="CBOX GR1A"></span>').replace('BOXGR2A','<span class="CBOX GR2A"></span>').replace('BOXGR2B','<span class="CBOX GR2B"></span>').replace('BOXGR3A','<span class="CBOX GR3A"></span>').replace('BOXGR3B','<span class="CBOX GR3B"></span>').replace('BOXGR3C','<span class="CBOX GR3C"></span>')
		sdeschtml = sdesc.replace('BOXGR1A','<span class="CBOX GR1A"></span>').replace('BOXGR2A','<span class="CBOX GR2A"></span>').replace('BOXGR2B','<span class="CBOX GR2B"></span>').replace('BOXGR3A','<span class="CBOX GR3A"></span>').replace('BOXGR3B','<span class="CBOX GR3B"></span>').replace('BOXGR3C','<span class="CBOX GR3C"></span>')
		charts.append((id,oname,desc,fdeschtml,sdeschtml))
	servers = []
	entrystr = []
	entrydesc = {}
	if len(hostlist)>0:
		for id,oname,desc,fdesc,sdesc in charts:
			name = oname.replace(":","")
			entrystr.append(name)
			entrydesc[name] = desc
		for ip,port,version in hostlist:
			if version>=(2,0,15):
				chmode = 10 if version>=(4,31,0) else 12
				strip = "%u.%u.%u.%u" % ip
				name = "%s:%u" % (strip,port)
				namearg = "%s:%u" % (name,chmode)
				hostx = resolve(strip,0,UNRESOLVED)
				if hostx==UNRESOLVED:
					host = ""
				else:
					host = " / "+hostx
				entrystr.append(namearg)
				entrydesc[namearg] = "Server: %s%s" % (name,host)
				servers.append((strip,port,"cs_"+name.replace(".","_").replace(":","_"),"Server: <b>%s%s</b>" % (name,host),chmode))

	cchtmp = CCdata.split(":")
	if len(cchtmp)==2:
		cchtmp = (cchtmp[0],cchtmp[1],0)
	if len(cchtmp)==3:
		cshost = cchtmp[0]
		csport = cchtmp[1]
		csmode = int(cchtmp[2])
		if csmode<10:
			csmode = 1
		else:
			csmode -= 10

		out.append("""<div class="tab_title">Chunkservers charts</div>""")
		out.append("""<form action="#"><table class="FR">""")
		out.append("""	<tr>""")
		out.append("""		<th class="knob-cell chart-range">""")
		out.append(html_knob_selector_chart_range('cs_main'))
		out.append("""		</th>""")
		out.append("""		<th class="chart-select">""")
		out.append("""			<div class="select-fl"><select class="chart-select" name="csdata" size="1" onchange="cs_change_data(this.selectedIndex,this.options[this.selectedIndex].value)">""")
		if CCdata not in entrystr:
			out.append("""				<option value="" selected="selected"> data type or server</option>""")
		for estr in entrystr:
			if estr==CCdata:
				out.append("""<option value="%s" selected="selected">%s</option>""" % (estr,entrydesc[estr]))
			else:
				out.append("""<option value="%s">%s</option>""" % (estr,entrydesc[estr]))
		out.append("""			</select><span class="arrow"></span></div>""")
		out.append("""		</th>""")
		out.append("""	</tr>""")
		if CCdata in entrystr:
			for id,name,desc,fdesc,sdesc in charts:
				out.append("""	<tr class="C2 CHART">""")
				out.append("""		<td id="cs_%s_p" colspan="3" style="height:124px;" valign="middle">""" % name)
				out.append("""			<div class="CHARTJSW">""")
				out.append("""				<div id="cs_%s_c" class="CHARTJSC">""" % name)
				out.append("""					<span class="chartcaptionjs">%s</span>""" % fdesc)
				out.append("""				</div>""")
				out.append("""			</div>""")
				out.append("""			<div id="cs_%s_l" class="CHARTJSL"></div>""" % name)
				out.append("""			<div id="cs_%s_r" class="CHARTJSR"></div>""" % name)
				out.append("""		</td>""")
				out.append("""	</tr>""")
		out.append("""</table></form>""")

		if CCdata in entrystr:
			out.append("""<div class="tab_title">Chunkserver charts (comparison)</div>""")
			out.append("""<form action="#"><table class="FR">""")
			for i in range(2):
				out.append("""	<tr>""")
				if i==0:
					out.append("""		<th class="knob-cell chart-cmp-range">""")
					out.append(html_knob_selector_chart_range('cs_cmp')) 
				else:
					out.append("""		<th style="text-align: right;vertical-align:middle;border:none;">""")
					out.append("""versus:""")
				out.append("""		</th>""")
				out.append("""		<th class="chart-cmp-select">""")
				out.append("""			<div class="select-fl"><select class="chart-select" id="cschart%u_select" name="cschart%u" size="1" onchange="cs_change_type(%u,this.options[this.selectedIndex].value)">""" % (i,i,i))
				no = 0
				for id,name,desc,fdesc,sdesc in charts:
					out.append("""				<option value="%u">%s</option>""" % (no,desc))
					no += 1
				out.append("""			</select><span class="arrow"></span></div>""")
				out.append("""		</th>""")
				out.append("""	</tr>""")
				out.append("""	<tr class="C2 CHART">""")
				out.append("""		<td id="cs_%u_p" colspan="2" style="height:124px; border-top: none;" valign="middle">""" % i)
				out.append("""			<div class="CHARTJSW">""")
				out.append("""				<div id="cs_%u_c" class="CHARTJSC">""" % i)
				out.append("""					<span id="cs_%u_d" class="chartcaptionjs">%s</span>""" % (i,charts[0][3]))
				out.append("""				</div>""")
				out.append("""			</div>""")
				out.append("""			<div id="cs_%u_l" class="CHARTJSL"></div>""" % i)
				out.append("""			<div id="cs_%u_r" class="CHARTJSR"></div>""" % i)
				out.append("""		</td>""")
				out.append("""	</tr>""")
			out.append("""</table></form>""")

			out.append("""<script type="text/javascript">""")
			out.append("""<!--//--><![CDATA[//><!--""")
			out.append("""	var cs_vids = [%s];""" % ",".join(map(repr,[ x[0] for x in charts ])))
			out.append("""	var cs_inames = [%s];""" % ",".join(map(repr,[ x[1] for x in charts ])))
			out.append("""	var cs_idesc = [%s];""" % ",".join(map(repr,[ x[3] for x in charts ])))
			out.append("""	var cs_hosts = [%s];""" % ",".join(map(repr,[ x[0] for x in servers ])))
			out.append("""	var cs_ports = [%s];""" % ",".join(map(repr,[ x[1] for x in servers ])))
			out.append("""	var cs_modes = [%s];""" % ",".join(map(repr,[ x[4]-10 for x in servers ])))
			out.append("""	var cs_host = "%s";""" % cshost)
			out.append("""	var cs_port = "%s";""" % csport)
			out.append("""	var cs_mode = %u;""" % csmode)
			out.append("""	var cs_valid = 1;""")
			out.append("""	var cs_base_href = "%s";""" % fields.createrawlink({"CCdata":""}))
			out.append("""//--><!]]>""")
			out.append("""</script>""")
		else:
			out.append("""<script type="text/javascript">""")
			out.append("""<!--//--><![CDATA[//><!--""")
			out.append("""	var cs_valid = 0;""")
			out.append("""	var cs_base_href = "%s";""" % fields.createrawlink({"CCdata":""}))
			out.append("""//--><!]]>""")
			out.append("""</script>""")
		out.append("""<script type="text/javascript">
<!--//--><![CDATA[//><!--
function cs_create_charts(show_loading=true) {
var i;
cs_charttab = [];
if (cs_valid) {
for (i=0 ; i<cs_inames.length ; i++) {
	cs_charttab.push(new AcidChartWrapper("cs_"+cs_inames[i]+"_","cs_main",cs_host,cs_port,cs_mode,cs_vids[i],show_loading));
}
if (cs_chartcmp[0]) document.getElementById('cschart0_select').value = cs_vids.indexOf(cs_chartcmp[0].id);
if (cs_chartcmp[1]) document.getElementById('cschart1_select').value = cs_vids.indexOf(cs_chartcmp[1].id);
cs_chartcmp[0] = new AcidChartWrapper("cs_0_","cs_cmp",cs_host,cs_port,cs_mode,(cs_chartcmp[0]) ? cs_chartcmp[0].id : cs_vids[0],show_loading);
cs_chartcmp[1] = new AcidChartWrapper("cs_1_","cs_cmp",cs_host,cs_port,cs_mode,(cs_chartcmp[1]) ? cs_chartcmp[1].id : cs_vids[0],show_loading);
}
}

// main charts
function cs_change_data(indx, ccdata) {
var sindx,i;
if (cs_valid) {
sindx = indx - cs_inames.length;
if (sindx >= 0 && sindx < cs_modes.length) {
	for (i=0 ; i<cs_vids.length ; i++) {
		cs_charttab[i].set_host_port_id(cs_hosts[sindx],cs_ports[sindx],cs_modes[sindx],cs_vids[i]);
	}
}
}
document.location.replace(cs_base_href + "&CCdata=" + ccdata);		   
}
// compare charts
function cs_change_type(chartid,indx) {
var descel;
if (cs_valid) {
cs_chartcmp[chartid].set_id(cs_vids[indx]);
descel = document.getElementById("cs_"+chartid+"_d");
descel.innerHTML = cs_idesc[indx];
}
}
cs_charttab = [];
cs_chartcmp = [];
cs_create_charts();
//--><!]]>
</script>""")
	elif len(cchtmp)==1 and len(CCdata)>0:
		chid = 0
		sdescadd = ''
		for id,name,desc,fdesc,sdesc in charts:
			if name==CCdata:
				chid = id
				if sdesc!='':
					sdescadd = ', '+sdesc
		if chid==0:
			try:
				chid = int(CCdata)
			except Exception:
				pass
		if chid<=0 or chid>=1000:
			CCdata = ""

		out.append("""<div class="tab_title">Chunkservers charts</div>""")
		out.append("""<form action="#"><table class="FR">""")
		out.append("""	<tr>""")
		out.append("""		<th class="knob-cell chart-range">""")
		out.append(html_knob_selector_chart_range('cs_main'))
		out.append("""		</th>""")
		out.append("""		<th class="chart-select">""")
		out.append("""			<div class="select-fl"><select class="chart-select" name="csdata" size="1" onchange="cs_change_data(this.selectedIndex,this.options[this.selectedIndex].value)">""")
		if CCdata not in entrystr:
			out.append("""				<option value="" selected="selected"> data type or server</option>""")
		for estr in entrystr:
			if estr==CCdata:
				out.append("""<option value="%s" selected="selected">%s</option>""" % (estr,entrydesc[estr]))
			else:
				out.append("""<option value="%s">%s</option>""" % (estr,entrydesc[estr]))
		out.append("""			</select><span class="arrow"></span></div>""")
		if CCdata in entrystr:
			out.append("""		<label class="switch" for="cs_commonscale" id="cs_commonscale-slide" style="margin-left:10px;"><input type="checkbox" id="cs_commonscale" onchange="AcidChartSetCommonScale('cs_main',this.checked)"/><span class="slider round"></span><span class="text">Use common Y-scale</span></label>""")				
		out.append("""		</th>""")
		out.append("""	</tr>""")
		if CCdata in entrystr:
			for cshost,csport,name,desc,chmode in servers:
				out.append("""	<tr class="C2 CHART">""")
				out.append("""		<td id="cs_%s_p" colspan="3" style="height:124px;" valign="middle">""" % name)
				out.append("""			<div class="CHARTJSW">""")
				out.append("""				<div id="cs_%s_c" class="CHARTJSC">""" % name)
				out.append("""					<span id="cs_%s_d" class="chartcaptionjs">%s%s</span>""" % (name,desc,sdescadd))
				out.append("""				</div>""")
				out.append("""			</div>""")
				out.append("""			<div id="cs_%s_l" class="CHARTJSL"></div>""" % name)
				out.append("""			<div id="cs_%s_r" class="CHARTJSR"></div>""" % name)
				out.append("""		</td>""")
				out.append("""	</tr>""")
		out.append("""</table></form>""")

		if CCdata in entrystr:
			out.append("""<script type="text/javascript">""")
			out.append("""<!--//--><![CDATA[//><!--""")
			out.append("""	var cs_vids = [%s];""" % ",".join(map(repr,[ x[0] for x in charts ])))
			out.append("""	var cs_idesc = [%s];""" % ",".join(map(repr,[ x[4] for x in charts ])))
			out.append("""	var cs_hosts = [%s];""" % ",".join(map(repr,[ x[0] for x in servers ])))
			out.append("""	var cs_ports = [%s];""" % ",".join(map(repr,[ x[1] for x in servers ])))
			out.append("""	var cs_inames = [%s];""" % ",".join(map(repr,[ x[2] for x in servers ])))
			out.append("""	var cs_sdesc = [%s];""" % ",".join(map(repr,[ x[3] for x in servers ])))
			out.append("""	var cs_modes = [%s];""" % ",".join(map(repr,[ x[4]-10 for x in servers ])))
			out.append("""	var cs_chid = %u;""" % chid)
			out.append("""	var cs_valid = 1;""")
			out.append("""	var cs_base_href = "%s";""" % fields.createrawlink({"CCdata":""}))
			out.append("""//--><!]]>""")
			out.append("""</script>""")
		else:
			out.append("""<script type="text/javascript">""")
			out.append("""<!--//--><![CDATA[//><!--""")
			out.append("""	var cs_valid = 0;""")
			out.append("""	var cs_base_href = "%s";""" % fields.createrawlink({"CCdata":""}))
			out.append("""//--><!]]>""")
			out.append("""</script>""")

		out.append("""<script type="text/javascript">
<!--//--><![CDATA[//><!--
function cs_create_charts(show_loading=true) {
var i;
cs_charttab = [];
if (cs_valid) {
for (i=0 ; i<cs_inames.length ; i++) {
	cs_charttab.push(new AcidChartWrapper("cs_"+cs_inames[i]+"_","cs_main",cs_hosts[i],cs_ports[i],cs_modes[i],cs_chid,show_loading));
}
}
}
// main charts
function cs_change_data(indx, ccdata) {
var i,chartid;
var descel;
if (cs_valid && indx >= 0 && indx < cs_vids.length) {
for (i=0 ; i<cs_inames.length ; i++) {
	chartid = cs_inames[i];
	cs_charttab[i].set_id(cs_vids[indx]);
	descel = document.getElementById("cs_"+chartid+"_d");
	if (cs_idesc[indx]!="") {
		descel.innerHTML = cs_sdesc[i] + " " + cs_idesc[indx];
	} else {
		descel.innerHTML = cs_sdesc[i];
	}
}
}
document.location.replace(cs_base_href + "&CCdata=" + ccdata);
}
cs_charttab = [];
cs_create_charts();
//--><!]]>
</script>""")
	else:
		out.append("""<div class="tab_title">Chunkservers charts</div>""")
		out.append("""<form action="#"><table class="FR">""")
		out.append("""	<tr>""")
		out.append("""		<th>""")
		out.append("""			Select: <div class="select-fl"><select class="chart-select" name="csdata" size="1" onchange="document.location.href='%s&CCdata='+this.options[this.selectedIndex].value">""" % fields.createrawlink({"CCdata":""}))
		if CCdata not in entrystr:
			out.append("""				<option value="" selected="selected"> data type or server</option>""")
		for estr in entrystr:
			if estr==CCdata:
				out.append("""<option value="%s" selected="selected">%s</option>""" % (estr,entrydesc[estr]))
			else:
				out.append("""<option value="%s">%s</option>""" % (estr,entrydesc[estr]))
		out.append("""			</select><span class="arrow"></span></div>""")
		out.append("""		</th>""")
		out.append("""	</tr>""")
		out.append("""</table></form>""")

	return out


from common.utils import *
from common.utilsgui import *

def render(dp, fields, vld):
	MOorder = fields.getint("MOorder", 0)
	MOrev = fields.getint("MOrev", 0)
	MOdata = fields.getint("MOdata", 0)

	out = []
	out.append("""<div class="tab_title">Active mounts (operations)</div>""")
	out.append("""<table class="acid_tab acid_tab_zebra_C1_C2 acid_tab_storageid_mfsops" id="mfsops">""")
	out.append("""	<tr>""")
	out.append("""		<th colspan="4" class="knob-cell">""")
	options=[(-90,"Last hour",None,"acid_tab.switchdisplay('mfsops','opshour_vis',0);"),
			(90, "Current hour",None,"acid_tab.switchdisplay('mfsops','opshour_vis',1);")]
	out.append(html_knob_selector("opshour_vis",9,(210,22),(100,11),options))

	out.append("""		</th>""")
	out.append("""		<th colspan="%u" style="vertical-align: middle;">""" % (1+dp.stats_to_show))
	out.append("""			<span class="opshour_vis0">Last hour operations</span>""")
	out.append("""			<span class="opshour_vis1">Current hour operations</span>""")
	out.append("""		</th>""")
	out.append("""	</tr>""")
	out.append("""	<tr>""")
	out.append("""		<th rowspan="1" class="acid_tab_enumerate">#</th>""")
	out.append("""		<th rowspan="1">Host</th>""")
	out.append("""		<th rowspan="1">IP</th>""")
	out.append("""		<th rowspan="1">Mount&nbsp;point</th>""")
	out.append("""		<th class="acid_tab_level_1">statfs</th>""")
	out.append("""		<th class="acid_tab_level_1">getattr</th>""")
	out.append("""		<th class="acid_tab_level_1">setattr</th>""")
	out.append("""		<th class="acid_tab_level_1">lookup</th>""")
	out.append("""		<th class="acid_tab_level_1">mkdir</th>""")
	out.append("""		<th class="acid_tab_level_1">rmdir</th>""")
	out.append("""		<th class="acid_tab_level_1">symlink</th>""")
	out.append("""		<th class="acid_tab_level_1">readlink</th>""")
	out.append("""		<th class="acid_tab_level_1">mknod</th>""")
	out.append("""		<th class="acid_tab_level_1">unlink</th>""")
	out.append("""		<th class="acid_tab_level_1">rename</th>""")
	out.append("""		<th class="acid_tab_level_1">link</th>""")
	out.append("""		<th class="acid_tab_level_1">readdir</th>""")
	out.append("""		<th class="acid_tab_level_1">open</th>""")
	out.append("""		<th class="acid_tab_level_1">rchunk</th>""")
	out.append("""		<th class="acid_tab_level_1">wchunk</th>""")
	if (dp.stats_to_show>16):
		out.append("""		<th class="acid_tab_level_1">read</th>""")
		out.append("""		<th class="acid_tab_level_1">write</th>""")
		out.append("""		<th class="acid_tab_level_1">fsync</th>""")
		out.append("""		<th class="acid_tab_level_1">snapshot</th>""")
		out.append("""		<th class="acid_tab_level_1">truncate</th>""")
		out.append("""		<th class="acid_tab_level_1">getxattr</th>""")
		out.append("""		<th class="acid_tab_level_1">setxattr</th>""")
		out.append("""		<th class="acid_tab_level_1">getfacl</th>""")
		out.append("""		<th class="acid_tab_level_1">setfacl</th>""")
		out.append("""		<th class="acid_tab_level_1">create</th>""")
		out.append("""		<th class="acid_tab_level_1">lock</th>""")
		out.append("""		<th class="acid_tab_level_1">meta</th>""")
	out.append("""		<th class="acid_tab_level_1">total</th>""")
	out.append("""	</tr>""")

	for ses in dp.get_sessions_order_by_mo(MOorder,MOrev,MOdata):
		out.append("""	<tr>""")
		out.append("""		<td align="right"></td>""")
		out.append("""		<td align="left">%s</td>""" % ses.host)
		out.append("""		<td align="center"><span class="sortkey">%s</span>%s</td>""" % (ses.sortip,ses.strip))
		out.append("""		<td align="left">%s</td>""" % htmlentities(ses.info))
		for st in range(dp.stats_to_show):
			out.append("""		<td align="right">""")
			out.append("""			<span class="opshour_vis0"><a style="cursor:default" title="current:%u last:%u">%s</a></span>""" % (ses.stats_c[st],ses.stats_l[st],decimal_number_html(ses.stats_l[st])))
			out.append("""			<span class="opshour_vis1"><a style="cursor:default" title="current:%u last:%u">%s</a></span>""" % (ses.stats_c[st],ses.stats_l[st],decimal_number_html(ses.stats_c[st])))
			out.append("""		</td>""")
		out.append("""		<td align="right">""")
		out.append("""			<span class="opshour_vis0"><a style="cursor:default" title="current:%u last:%u">%s</a></span>""" % (sum(ses.stats_c),sum(ses.stats_l),decimal_number_html(sum(ses.stats_l))))
		out.append("""			<span class="opshour_vis1"><a style="cursor:default" title="current:%u last:%u">%s</a></span>""" % (sum(ses.stats_c),sum(ses.stats_l),decimal_number_html(sum(ses.stats_c))))
		out.append("""		</td>""")
		out.append("""	</tr>""")

	out.append("""</table>""")
	return out